use serde::{Deserialize, Serialize};
use std::fs;
use std::path::PathBuf;

#[derive(Serialize, Deserialize, Debug)]
pub struct Config {
    pub explorer: Option<String>,
    pub base_path: Option<String>,
}

pub fn get_config_path() -> Result<PathBuf, String> {
    let config_dir = dirs::config_dir().ok_or_else(|| "Could not find config dir".to_string())?;
    let app_config_dir = config_dir.join("ionian-find");
    fs::create_dir_all(&app_config_dir)
        .map_err(|e| format!("Could not create config dir: {}", e))?;
    Ok(app_config_dir.join("config.json"))
}

pub fn load_config() -> Result<Config, String> {
    let config_path = get_config_path()?;
    let content = fs::read_to_string(config_path)
        .map_err(|e| format!("Could not read config file: {}", e))?;
    serde_json::from_str(&content).map_err(|e| format!("Could not parse config file: {}", e))
}

pub fn save_config(config: &Config) -> Result<(), String> {
    let config_path = get_config_path()?;
    let content = serde_json::to_string_pretty(config)
        .map_err(|e| format!("Could not serialize config: {}", e))?;
    fs::write(config_path, content).map_err(|e| format!("Could not write config file: {}", e))
}
