use std::env;
use std::fs;

use std::process::Command;

pub fn handle_register() -> Result<(), String> {
    println!("Registering the deep link for Linux...");

    let exe_path = env::current_exe().map_err(|e| format!("Error getting current executable path: {}", e))?;

    let desktop_entry = format!(
        "[Desktop Entry]\n\
         Type=Application\n\
         Name=Ionian Find\n\
         Exec={} open %u\n\
         StartupNotify=false\n\
         MimeType=x-scheme-handler/ionian-find;\n",
        exe_path.to_str().unwrap_or("")
    );

    let data_local_dir = dirs::data_local_dir().ok_or("Could not find local data directory.")?;

    let app_dir = data_local_dir.join("applications");
    fs::create_dir_all(&app_dir).map_err(|e| format!("Error creating directory: {}", e))?;

    let desktop_file_path = app_dir.join("ionian-find.desktop");

    fs::write(&desktop_file_path, desktop_entry).map_err(|e| format!("Error writing desktop file: {}", e))?;

    // Update the desktop database
    Command::new("update-desktop-database")
        .arg(app_dir.to_str().unwrap())
        .output()
        .map_err(|e| format!("Failed to execute update-desktop-database: {}", e))?;

    let output = Command::new("xdg-mime")
        .args(&[
            "default",
            "ionian-find.desktop",
            "x-scheme-handler/ionian-find",
        ])
        .output()
        .map_err(|e| format!("Failed to execute xdg-mime: {}", e))?;

    if output.status.success() {
        println!("Deep link registered successfully!");
    } else {
        return Err(format!(
            "Error registering deep link: {}\n{}",
            String::from_utf8_lossy(&output.stdout),
            String::from_utf8_lossy(&output.stderr)
        ));
    }

    Ok(())
}

pub fn handle_unregister() -> Result<(), String> {
    println!("Unregistering the deep link for Linux...");

    let output = Command::new("xdg-mime")
        .args(&[
            "default",
            "org.gnome.Nautilus.desktop",
            "x-scheme-handler/ionian-find",
        ])
        .output()
        .map_err(|e| format!("Failed to execute xdg-mime: {}", e))?;

    if output.status.success() {
        println!("Deep link unregistered successfully!");
    } else {
        return Err(format!(
            "Error unregistering deep link: {}\n{}",
            String::from_utf8_lossy(&output.stdout),
            String::from_utf8_lossy(&output.stderr)
        ));
    }

    Ok(())
}
