use std::env;
use winreg::enums::*;
use winreg::RegKey;

const PATH: &str = r"Software\Classes\ionian-find";

pub fn handle_register() -> Result<(), String> {
    println!("Registering the deep link for Windows...");

    let exe_path = env::current_exe()
        .map_err(|e| format!("Error getting current executable path: {}", e))?
        .to_str()
        .ok_or("Invalid UTF-8 in executable path")?
        .to_string();

    let hkcu = RegKey::predef(HKEY_CURRENT_USER);
    let (key, disp) = hkcu.create_subkey(&PATH)
        .map_err(|e| format!("Failed to create subkey: {}", e))?;

    match disp {
        REG_CREATED_NEW_KEY => println!("Created new registry key"),
        REG_OPENED_EXISTING_KEY => println!("Opened existing registry key"),
    }

    key.set_value("", &"URL:ionian-find")
        .map_err(|e| format!("Failed to set value: {}", e))?;
    key.set_value("URL Protocol", &"")
        .map_err(|e| format!("Failed to set value: {}", e))?;

    let (shell_key, _) = key.create_subkey("shell")
        .map_err(|e| format!("Failed to create subkey: {}", e))?;
    let (open_key, _) = shell_key.create_subkey("open")
        .map_err(|e| format!("Failed to create subkey: {}", e))?;
    let (command_key, _) = open_key.create_subkey("command")
        .map_err(|e| format!("Failed to create subkey: {}", e))?;

    let command_value = format!(r#""{}" "open" "%1""#, exe_path);
    command_key.set_value("", &command_value)
        .map_err(|e| format!("Failed to set value: {}", e))?;

    println!("Deep link registered successfully!");
    Ok(())
}

pub fn handle_unregister() -> Result<(), String> {
    println!("Unregistering the deep link for Windows...");

    let hkcu = RegKey::predef(HKEY_CURRENT_USER);

    hkcu.delete_subkey_all(PATH)
        .map_err(|e| format!("Failed to delete subkey: {}", e))?;

    println!("Deep link unregistered successfully!");
    Ok(())
}